#!/usr/bin/env python3

import os
import subprocess


class WorkspaceFormatter:
    @staticmethod
    def format_workspace(lines):
        if "splith" in lines[2]:
            lines = lines[2:]
            lines[0] = "{\n"
        else:
            lines = lines[1:]

        return "".join(str(line) for line in lines)

    @classmethod
    def format_and_overwrite(cls, workspaces):
        for i in range(0, 10):
            path = f"{workspaces}/workspace_{i}.json"
            with open(path) as file:
                try:
                    lines = file.readlines()

                    if len(lines) <= 1:
                        continue

                    result = cls.format_workspace(lines).replace("//", "")
                except FileNotFoundError:
                    raise Exception(f"Workspace {i}'s config not found!")
            with open(path, "w") as file:
                file.write(result)

                file.close()

    @staticmethod
    def load_workspaces(workspaces):
        dirname = os.path.dirname(__file__)
        sc_path = os.path.join(dirname, "./bin/load-workspaces.sh")
        subprocess.run(
            f"{sc_path} {workspaces}",
            shell=True,
        )

    @classmethod
    def save_workspaces(cls, workspaces):
        dirname = os.path.dirname(__file__)
        sc_path = os.path.join(dirname, "./bin/save-workspaces.sh")
        subprocess.run(f"{sc_path} {workspaces}", shell=True)

        cls.format_and_overwrite(workspaces)



#!/usr/bin/env python3

import argparse
import os


class Args:
    def __init__(self):
        self.parser = argparse.ArgumentParser(prog="PROG")
        subparsers = self.parser.add_subparsers(dest="method")

        # organize
        parser_workspace = subparsers.add_parser(
            "organize",
            help="Apply backed up i3wm workspaces or backup i3wm workspaces.",
        )
        parser_workspace.add_argument(
            "-a", "--action", choices=["load", "save"], required=True
        )
        parser_workspace.add_argument("-w", "--workspaces", type=str, required=True)

        # lockscreen
        subparsers.add_parser("lockscreen", help="Prevents lockscreen execution.")

        # toggle
        parser_toggle = subparsers.add_parser("toggle", help="Toggles monitor.")
        parser_toggle.add_argument("-mn", "--monitor-number", type=int, required=True)
        parser_toggle.add_argument("-lt", "--locate-to", choices=["left", "right"])
        parser_toggle.add_argument("-lo", "--locate-of", type=int)

        args = self.parser.parse_args()
        method = args.method

        self.method = method

        match method:
            case "organize":
                self.action = args.action
                self.workspaces = os.path.abspath(args.workspaces)
            case "lockscreen":
                return
            case "toggle":
                self.monitor_number = args.monitor_number - 1
                self.locate_to = args.locate_to
                self.locate_of = args.locate_of - 1
            case _:
                self.parser.error("Please provide valid operation method.")



from src.i3_screen_util.args import Args
from src.i3_screen_util.monitor_controller import BACKUP_FILE_PATH
from src.i3_screen_util import run_app

import sys
import subprocess
from pathlib import Path


def test_organize_method():
    tmp = sys.argv
    try:
        sys.argv = [
            tmp[0],
            "organize",
            "--action",
            "save",
            "--workspaces",
            "./tmp",
        ]

        args = Args()
        run_app()

        assert args.action == "save"

        for i in range(10):
            workspace_path = Path(f"{args.workspaces}/workspace_{i}.json")
            assert workspace_path.exists() is True

        subprocess.run(f"rm -rf {args.workspaces}", shell=True)

        # TODO: add load workspace test suite as well
    finally:
        sys.argv = tmp


def test_lockscreen_method():
    tmp = sys.argv
    try:
        sys.argv = [
            tmp[0],
            "lockscreen",
        ]

        run_app()

        running_process = subprocess.run(
            "ps -aux | grep control-lockscreen | grep -v grep | awk '{ print $2 }'",
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )

        running_process = running_process.stdout.decode()

        assert len(running_process) > 0

        subprocess.run(
            "ps -aux | grep control-lockscreen | grep -v grep | awk '{ print $2 }' | xargs kill -9",
            shell=True,
        )
    finally:
        sys.argv = tmp


def test_toggle_method():
    tmp = sys.argv
    try:
        sys.argv = [
            tmp[0],
            "toggle",
            "--monitor-number",
            "2",
            "--locate-to",
            "left",
            "--locate-of",
            "1",
        ]

        run_app()
        args = Args()

        assert args.monitor_number == 1
        assert args.locate_to == "left"
        assert args.locate_of == 0

        backup_file_path = Path(BACKUP_FILE_PATH)

        assert backup_file_path.exists() is True
    finally:
        sys.argv = tmp



#!/usr/bin/env python3

import json
import os
import re
import subprocess
import sys

BACKUP_FILE_PATH = "/var/tmp/i3-screen-util.data.bin"

# reference: https://www.thinkwiki.org/wiki/Xorg_RandR_1.2#Output_port_names
XRANDR_DISPLAY_TYPES = [
    "VGA",
    "LVDS",
    "DP1",
    "TV",
    "TMDS-1",
    "TMDS-2",
    "LVDS1",
    "VGA1",
    "DVI1",
    "VGA-0",
    "LVDS",
    "S-video",
    "DVI-0",
    "HDMI",
    "DVI",
    "DP",
]


class Backup:
    @staticmethod
    def get_backup_data(options):
        with open(BACKUP_FILE_PATH, "r") as backup_file:
            backup_data = backup_file.read()

            if len(backup_data) == 0:
                data = {}
                for option in options:
                    data[str(option)] = True
                with open(BACKUP_FILE_PATH, "w") as backup_file:
                    tmp = json.dumps(data)
                    backup_file.write(
                        " ".join(format(ord(letter), "b") for letter in tmp)
                    )
                    backup_file.close()
                return data
            else:
                data_raw = "".join(chr(int(x, 2)) for x in backup_data.split())
                return json.loads(data_raw)

    @staticmethod
    def save_backup_data(new_backup_data):
        with open(BACKUP_FILE_PATH, "w") as backup_file:
            tmp = json.dumps(new_backup_data)
            backup_file.write(" ".join(format(ord(letter), "b") for letter in tmp))
            backup_file.close()


class MonitorController:
    @staticmethod
    def turn_off_monitor(monitor_name, backup_data):
        subprocess.run(f"xrandr --output {monitor_name} --off", shell=True)

        backup_data[monitor_name] = False
        Backup.save_backup_data(backup_data)

    @staticmethod
    def turn_on_monitor(monitor_name, to, side_monitor_name, backup_data):
        subprocess.run(
            f"xrandr --output {monitor_name} --auto --{to}-of {side_monitor_name}",
            shell=True,
        )

        backup_data[monitor_name] = True
        Backup.save_backup_data(backup_data)

    @classmethod
    def toggle_monitor(cls, monitor_number, locate_to, locate_of):
        if os.path.exists(BACKUP_FILE_PATH) is False:
            with open(BACKUP_FILE_PATH, "w") as backup_file:
                backup_file.close()
                pass

        monitors_raw = subprocess.run(
            "xrandr", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        # wow i can do regex search UwU
        pattern = r"([A-Za-z]+-\d+) connected"
        displays = re.findall(pattern, monitors_raw.stdout.decode())
        options = [
            display
            for display in displays
            if any(display.startswith(dt) for dt in XRANDR_DISPLAY_TYPES)
        ]

        try:
            monitor_name = options[monitor_number]
            side_monitor_name = None
            if locate_of is not None:
                side_monitor_name = options[locate_of]

            backup_data = Backup.get_backup_data(options)

            is_turned_on = backup_data[monitor_name]

            if is_turned_on is True:
                cls.turn_off_monitor(monitor_name, backup_data)
            else:
                if locate_to is None or side_monitor_name is None:
                    raise Exception(
                        "'locate_to' or 'location_of' arguments cannot be None."
                    )

                cls.turn_on_monitor(
                    monitor_name, locate_to, side_monitor_name, backup_data
                )
        except IndexError:
            raise Exception("Invalid monitor number.")
        except Exception:
            sys.exit(0)



#!/usr/bin/env python3

from .args import Args
from .lockscreen import Lockscreen
from .monitor_controller import MonitorController
from .workspace_formatter import WorkspaceFormatter


def run_app():
    args = Args()

    match args.method:
        case "organize":
            if args.action == "load":
                WorkspaceFormatter.load_workspaces(args.workspaces)
            elif args.action == "save":
                WorkspaceFormatter.save_workspaces(args.workspaces)
        case "lockscreen":
            Lockscreen.control_lockscreen()
        case "toggle":
            MonitorController.toggle_monitor(
                args.monitor_number, args.locate_to, args.locate_of
            )



#!/usr/bin/env python3

import os
import subprocess


class Lockscreen:
    @staticmethod
    def control_lockscreen():
        dirname = os.path.dirname(__file__)
        sc_path = os.path.join(dirname, "./bin/control-lockscreen.sh")
        subprocess.Popen(sc_path)
